#!/bin/bash -x
set -e
if [  $UID != 0 ]
then
	zenity --error --title "Creador de pendrive" --text="Es necesario ejecutarlo como super usuario"
	exit 20
fi

if [ $(basename `which pv`) != "pv" ]; then
	zenity --error --title "Creador de pendrive" --text="Es necesario el comando 'pv'"
	exit 5
fi
buscapincho(){
	# Buscamos los posibles pendrive, para ello usamos hal con un comando que implementa 
	# una busqueda por prestaciones, buscamos disositivos de almacenamiento (storage).
	for pen in $(hal-find-by-capability --capability storage)
	do
		hal-device $pen|
		grep -C70 "storage.drive_type = 'disk'"|
		grep -C70 "storage.removable = true"|
		grep -C70 "storage.removable.media_available = true"|
		grep block.device|
		cut -d"'" -f2
		#aqui comprobamos cuales de los dispositivos encontrados son pendrive, tambien evitamos que los lectores de sd sean considerados como dispositivos validos.
	done
}
desmonta() {
	# Desmontamos todas las particiones de un dispositivo concreto.
	# Modificamos {print $3 } por {print $1 } en el bucle "for"
	for i in $(mount|grep $1|awk '{print $1}' )
	do
		umount $i
	done
}

busca_grub(){	
	echo -e "default 0\ntimeout 10\ncolor cyan/blue white/blue\ncommand 'lock'\nbackground 222244\nsplashimage=/boot/grub/splash.xpm.gz\n" >> $2
	
	if $(grep -v "#" $1| grep -q "duro local")
	then
		echo -e "title Arrancar desde disco duro local\nrootnoverify (hd1)\nmap (hd1) (hd0)\nchainloader +1\n" >> $2
	fi
	if $(grep -v "#" $1|grep -q "10.04")
	then
		if $(grep -v "#" $1| grep -q "installge.cfg")
		then            
			echo -e "title Instalar GuadalinexEdu 10.04\nroot (hd0,0)\n$(grep -e "installge.cfg" $1| sed "s/linux\ \//kernel\ \//g" |tr -s ' ' ' '| sed "s/^\ //g")\ninitrd /boot/initrd.gz\n" >> $2
		fi	
		if $(grep -v "#" $1| grep -q "installgeaula20.cfg" )
		then
			echo -e "title Instalar GuadalinexEdu 10.04 escuela 2.0\nroot (hd0,0)\n$(grep -e "installgeaula20.cfg" $1| sed "s/linux\ \//kernel\ \//g" |tr -s ' ' ' '| sed "s/^\ //g")\ninitrd /boot/initrd.gz\n" >> $2
		fi
		if $(grep -v "#" $1| grep -q "installgeall.cfg")
		then            
			echo -e "title Instalar GuadalinexEdu 10.04 completa\nroot (hd0,0)\n$(grep -e "installgeall.cfg" $1| sed "s/linux\ \//kernel\ \//g" |tr -s ' ' ' '| sed "s/^\ //g")\ninitrd /boot/initrd.gz\n" >> $2
		fi	
		if $(grep -v "#" $1| grep -q "installgeaula20all.cfg" )
		then
			echo -e "title Instalar GuadalinexEdu 10.04 escuela 2.0 completa\nroot (hd0,0)\n$(grep -e "installgeaula20all.cfg" $1| sed "s/linux\ \//kernel\ \//g" |tr -s ' ' ' '| sed "s/^\ //g")\ninitrd /boot/initrd.gz\n" >> $2
		fi
	else
		if $(grep -v "#" $1| grep -q "Instalaedu" )
		then 
			echo -e "title Instalaedu\nroot (hd0,0)\n$(grep -e "installge.cfg" $1| sed "s/linux\ \//kernel\ \//g" |tr -s ' ' ' '| sed "s/^\ //g"|sed "s/\t//g")\ninitrd /boot/initrd.gz\n" >> $2
		fi
		if $(grep -v "#" $1| grep -q "Instalaedue20" )
		then
			echo -e "title Instalaedue20\nroot (hd0,0)\n$(grep -e "installge20.cfg" $1| sed "s/linux\ \//kernel\ \//g" |tr -s ' ' ' '| sed "s/^\ //g"|sed "s/\t//g")\ninitrd /boot/initrd.gz\n" >> $2
		fi
	fi
	}

if [ "$(buscapincho)" = "" ]
then
	zenity --error --title "Creador de pendrive" --text="Tiene que introducir un pendrive"
	exit 10
elif $(zenity --question --title "Creador de pendrive" --text="Advertencia:\nEl uso de esta aplicación implica el borrado del llavero usb utilizado.\n¿Desea continuar?") 
then
	PINCHO=$(buscapincho|zenity --list --title "Creador de pendrive" --column="unidades" --text="¿Qué pendrive usar?")
	if [ "$?" == 1 ] ; then exit 10; fi
	IMAGEN=$(zenity --file-selection --title="Selecciona la imagen del pendrive")
	if [ "$?" == 1 ] ; then exit 15; fi
else
	exit 123
fi
if [ $(fdisk -l $PINCHO|grep bytes|grep Disco|sed "s/^\ //g"|awk '{print $5}') -le $(du $IMAGEN|sed "s/^\ //g"|awk '{print $1}') ]
then
	zenity --error --title "Creador de pendrive" --text="Es necesario un pendrive de mayor tamaño"
	exit 129
fi
desmonta $PINCHO
LOOP=$(losetup -f)
if $(file $IMAGEN|grep -q "ISO 9660 CD-ROM")
then
	losetup $LOOP $IMAGEN
elif $(file $IMAGEN|grep -q "x86 boot sector")
then
	losetup $LOOP $IMAGEN
	EPART=$(sfdisk -d $LOOP |
		grep -e "Id= b" -e "Id= c"|
		sed "s/\ //g"|
		tr -s '=' ' '|
		awk '{print $2}'|
		cut -d',' -f1|
		head -n1)
	TSECTOR=$(fdisk -l $LOOP|
		  grep bytes|
		  awk '{print $7}'|
		  tail -n1)
	losetup -d $LOOP
	LOOP=$(losetup -f)
	losetup -o $(( $EPART * $TSECTOR )) $LOOP $IMAGEN
else
	zenity --error --title "Creador de pendrive" --text="Imagen de pendrive no valida"
	exit 142
fi
DEST=`pwd`/.destino
ORIG=`pwd`/.origen
if ! [ -e $DEST ]; then mkdir $DEST; fi
if ! [ -e $ORIG ]; then mkdir $ORIG; fi
mount $LOOP $ORIG
(dd if=/dev/zero of=$PINCHO bs=1024 count=1024; fdisk $PINCHO << EOF
n
p
1


t
c
w
EOF
mkdosfs -F 32 ${PINCHO}1
mount ${PINCHO}1 $DEST)|zenity --progress --title "Creador de pendrive" --text "iniciando pendrive" --pulsate --auto-close
rsync --exclude "repositorio.iso" -rP $ORIG/* $DEST|
while read i
do 
	cosa=$(echo $i|
	       grep to-check|
	       tr -s ' ' ' '|
	       cut -d'=' -f2|
	       cut -d')' -f1)
	total=$(echo $cosa|
		cut -d'/' -f2)
	parcial=$(echo $cosa|
		  cut -d'/' -f1)
	if [ $parcial ] && [ $total ]
	then 
		echo $(( ( $total - $parcial ) * 100 / $total ))
	fi
done|zenity --progress --title "Creador de pendrive" --text "Copiando componentes iniciales" --percentage=0 --auto-close
(pv -n $ORIG/repositorio.iso > $DEST/repositorio.iso) 2>&1 | zenity --progress --title "Creador de pendrive" --percentage=0 --auto-close --text "Completando proceso de copia"
(echo "finalizando"
sync
umount $ORIG
rmdir $ORIG
losetup -d $LOOP
sync
if $(find $DEST/boot/|grep -q "menu.lst")
then
	grub-install --root-directory=$DEST $PINCHO
elif $(find $DEST/boot/|grep -q "grub.cfg")
then
	GRUBCFG=$(find $DEST/boot/|grep "grub.cfg$"|grep -v i386-pc)
	convert -resize 640x480 -colors 14 $DEST/$(grep background_image $GRUBCFG|tr -s ' ' '\n'|grep boot|grep -v ".cfg:$") $DEST/boot/grub/splash.xpm
	gzip $DEST/boot/grub/splash.xpm
	busca_grub $GRUBCFG $DEST/boot/grub/menu.lst
	grub-install --root-directory=$DEST $PINCHO
fi
umount $DEST
rmdir $DEST
sync)|zenity --progress --title "Finalizando pendrive" --text "Espere por favor" --pulsate --auto-close
zenity --info --title "Creador de pendrive" --text "Proceso finalizado.\nDesconecte el pendrive."
exit 0
